package sound;

/**
 * Created by
 * User: lyon
 * Date: Nov 29, 2003
 * Time: 11:11:38 AM
 *
 */

import gui.run.RunScroll;

import javax.swing.JPanel;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridLayout;
import java.awt.Rectangle;
import java.awt.Scrollbar;
import java.awt.geom.Line2D;

public class DualOscopePanel
        extends JPanel {
    private RunScroll topSb;
    private RunScroll bottomSb;
    private RunScroll leftSb;
    private RunScroll rightSb;
    private RunScroll bottomSb2;
    private RunScroll leftSb2;
    private RunScroll rightSb2;
    private double[] trace1;
    private double[] trace2;
    private int dx = 0;
    private int dx2 = 0;
    private int oldDx = 0;
    private int oldDx2 = 0;
    private int dy = 0;
    private int dy2 = 0;
    private int oldDy = 0;
    private int oldDy2 = 0;
    private final double
            xScaleFactors[] = {50000, 25000, 10000, 5000, 2500, 1000,
                               500, 250, 100, 50, 25, 10,
                               5, 2.5, 1, .5, .25, .1,
                               .05, .025, .01, .005, .0025, .001,
                               .0005};
    private final String xSFLabels[] = {"0.05 u", "0.1 u", "0.25 u",
                                        "0.5 u", "1 u", "2.5 u",
                                        "5 u", "10 u", "25 u",
                                        "50 u", "100 u", "250 u",
                                        "500 u", "1 m", "2.5 m",
                                        "5 m", "10 m", "25 m",
                                        "50 m", "100 m", "250 m",
                                        "500 m", "1 ", "2.5 ",
                                        "5 "};
    private final int xsfStartIndex = 14;
    private double xScaleFactor = xScaleFactors[xsfStartIndex];
    private double oldXScaleFactor = xScaleFactors[xsfStartIndex];
    private String xSFLabel = new String(xSFLabels[xsfStartIndex]);
    private boolean labelsVisible = true;
    private String title = null;

    public void setLabelsVisible(boolean b) {
        labelsVisible = b;
    }

    public static final double
            yScaleFactors[] = {
                50000,
                25000,
                10000,
                5000,
                2000,
                1000,
                500,
                200,
                100,
                50,
                20,
                10,
                5, 2, 1, .5, .2, .1};
    public static final String
            ySFLabels[] = {
                "1 m", "2.5 m",
                "5 m", "10 m",
                "25 m", "50 m",
                "100 m", "250 m",
                "500 m",
                "1 ", "2.5 ",
                "5 ", "10 ",
                "20 ", "50 ",
                "100 ", "250 ", "500 "};
    private final int yScaleFactorStartIndex = 8;
    private double yScaleFactor = 1;
    private double yScaleFactor2 = 1;
    private double oldYScaleFactor = 1;
    private double oldYScaleFactor2 = 1;
    private String ySFLabel = new String(ySFLabels[yScaleFactorStartIndex]);
    private String ySFLabel2 = new String(ySFLabels[yScaleFactorStartIndex]);
    private TracePanel tracePanel = new TracePanel();

    /**
     * This constructor uses a 440 Hz sine wave
     * as the default waveform.
     */
    public DualOscopePanel() {
        this(new Oscillator(440, 1000).getSineWave(),
                "440 HZ sineWave");
    }

    public Dimension getPreferredSize() {
        return new Dimension(400, 200);
    }

    /**
     * A read only variable
     *
     * @return the title as set by the constructor
     */
    public String getTitle() {
        return title;
    }

    /**
     * This constructor uses the <code>doubleData</code>
     * as the source for the default waveform.
     *
     * @param doubleData an array of data to be displayed
     * @param title      title of the panel.
     */
    //public OscopePanel(double doubleData[], String title) {
    public DualOscopePanel(double doubleData[], String title) {
        this.trace1 = doubleData;
        this.trace2 = new Oscillator(220, 1000).getSineWave();
        this.title = title;
        setLayout(new BorderLayout());
        topSb =
                new RunScroll(Scrollbar.HORIZONTAL) {
                    public void run() {
                        int i = getValue();
                        xScaleFactor = xScaleFactors[i];
                        if (xScaleFactor != oldXScaleFactor) {
                            xSFLabel = xSFLabels[i];
                            tracePanel.repaint(500);
                            oldXScaleFactor = xScaleFactor;
                        }
                    }
                };
        // Set top scrollbar characteristics
        topSb.setValues(xsfStartIndex, 0, 0, 24);
        //topSb2.setValues(xsfStartIndex, 0, 0, 24);
        bottomSb2 =
                new RunScroll(Scrollbar.HORIZONTAL) {
                    public void run() {
                        dx2 = getValue();
                        if (dx2 != oldDx2) {
                            tracePanel.repaint(500);
                            oldDx2 = dx2;
                        }
                    }
                };
        bottomSb =
                new RunScroll(Scrollbar.HORIZONTAL) {
                    public void run() {
                        dx = getValue();
                        if (dx != oldDx) {
                            tracePanel.repaint(500);
                            oldDx = dx;
                        }
                    }
                };
        leftSb2 =
                new RunScroll(Scrollbar.VERTICAL) {
                    public void run() {
                        dy2 = getValue();
                        if (dy2 != oldDy2) {
                            tracePanel.repaint(500);
                            oldDy2 = dy2;
                        }
                    }
                };
        leftSb =
                new RunScroll(Scrollbar.VERTICAL) {
                    public void run() {
                        dy = getValue();
                        if (dy != oldDy) {
                            tracePanel.repaint(500);
                            oldDy = dy;
                        }
                    }
                };
        rightSb2 =
                new RunScroll(Scrollbar.VERTICAL) {
                    public void run() {
                        int i = getValue();
                        yScaleFactor2 = yScaleFactors[i];
                        if (yScaleFactor2 != oldYScaleFactor2) {
                            ySFLabel2 = ySFLabels[i];
                            tracePanel.repaint(500);
                            oldYScaleFactor2 = yScaleFactor2;
                        }
                    }
                };
        rightSb =
                new RunScroll(Scrollbar.VERTICAL) {
                    public void run() {
                        int i = getValue();
                        yScaleFactor = yScaleFactors[i];
                        if (yScaleFactor != oldYScaleFactor) {
                            ySFLabel = ySFLabels[i];
                            tracePanel.repaint(500);
                            oldYScaleFactor = yScaleFactor;
                        }
                    }
                };
        // Set right scrollbar characteristics
        rightSb.setValues(yScaleFactorStartIndex, 0, 0,
                yScaleFactors.length - 1);
        rightSb2.setValues(yScaleFactorStartIndex, 0, 0,
                yScaleFactors.length - 1);



        // Set left scrollbar characteristics
        leftSb.setValues(0, 0, -300, 300);
        leftSb2.setValues(0, 0, -300, 300);
        add(topSb, BorderLayout.NORTH);
        add(bottomSb, BorderLayout.SOUTH);
        add(leftSb, BorderLayout.WEST);
        add(rightSb, BorderLayout.EAST);
        add(tracePanel, BorderLayout.CENTER);
        setBackground(Color.white);

        // Set bottom scrollbar characteristics based on sample length
        bottomSb2.setValues(0, 0, 0, this.trace2.length);
        bottomSb.setValues(0, 0, 0, this.trace1.length);
    }

    public void setTrace1(double d[]) {
        trace1 = d;
    }

    public RunScroll getTopSb() {
        return topSb;
    }

    public RunScroll getBottomSb() {
        return bottomSb;
    }

    public RunScroll getBottomSb2() {
        return bottomSb2;
    }

    public RunScroll getLeftSb() {
        return leftSb;
    }

    public RunScroll getLeftSb2() {
        return leftSb2;
    }

    public RunScroll getRightSb() {
        return rightSb;
    }

    public RunScroll getRightSb2() {
        return rightSb2;
    }

    public Component getTracePanel() {
        return tracePanel;
    }

    public void setTopSb(RunScroll topSb) {
        this.topSb = topSb;
    }

    class TracePanel extends JPanel {
        private boolean drawGrid = true;
        private Color gridColor = new Color(0, 255, 0);
        private Dimension dim;
        private int height, width;

        public void setGridColor(Color c) {
            gridColor = c;
        }

        public void grid(int grid_spacing, Graphics g, Color c) {
            Rectangle r = g.getClipBounds();
            int w = r.width;
            int h = r.height;
            Color oldColor = g.getColor();
            g.setColor(c);
            for (int x = 0; x < w; x = x + grid_spacing) {
                g.drawLine(x, 0, x, h);
            }
            for (int y = 0; y < h; y = y + grid_spacing) {
                g.drawLine(0, y, w, y);
            }
            g.setColor(oldColor);
        }

        public void paint(Graphics g) {
            drawData(g);
            if (!labelsVisible) return;
            drawXScaleLabel(g);
            drawYScaleLabel2(g);
            drawYScaleLabel(g);
        }

        private void drawData(Graphics gOld) {
            Graphics2D g = (Graphics2D) gOld;
            double limit = (trace1.length * xScaleFactor);
            double limit2 = (trace2.length * xScaleFactor);
            dim = getSize();
            height = dim.height;
            width = dim.width;
            g.clearRect(0, 0, width, height);
            if (drawGrid)
                grid(20, g, gridColor);
            if (limit2 > width) limit2 = (double) width;
            int k = dx2;
            drawTrace2(limit2, k, g,trace2);
            if (limit > width) limit = (double) width;
            int j = dx;
            drawTrace1(limit, j, g);
        }

        private void drawTrace1(double limit, int j, Graphics2D g) {
            done:
            for (double i = 0; i < limit; i += xScaleFactor) {
                if (j >= (trace1.length - 1))
                    break done;
                double x1 = i;
                double y1 = (height - (
                        (trace1[j] * yScaleFactor)
                        * height / 4 + height / 2)) - dy;
                double x2 = (i + xScaleFactor);
                double y2 = (height - (
                        (trace1[j + 1] * yScaleFactor)
                        * height / 4 + height / 2)) - dy;
                g.draw(new Line2D.Double(x1, y1, x2, y2));
                g.setColor(Color.red);
                j++;
            }
        }

        private void drawTrace2(double limit2, int k, Graphics2D g, double d[]) {
            done:
            for (double i = 0; i < limit2; i += xScaleFactor) {
                if (k >= (trace2.length - 1))
                    break done;
                double x11 = i;
                double y11 = height - d[k] * yScaleFactor2 * height / 4 + height / 2 - dy2;
                double x22 = (i + xScaleFactor);
                double y22 = (height - (
                        (d[k + 1] * yScaleFactor2)
                        * height / 4 + height / 2)) - dy2;
                g.draw(new Line2D.Double(x11, y11, x22, y22));
                k++;
            }
        }

        private void drawXScaleLabel(Graphics g) {
            g.setColor(Color.black);
            int height = 10;
            int xloc = 20;
            int yloc = 20;
            String xsfLabel = xSFLabel + "sec/div";
            int string_width = getFontMetrics(g.getFont()).stringWidth(xsfLabel);
            int string_height = getFontMetrics(g.getFont()).getHeight();
            g.clearRect(xloc, yloc, string_width, string_height);
            g.drawString(xsfLabel, xloc, height + yloc);
        }

        private void drawYScaleLabel2(Graphics g) {
            int height = 10;
            int xloc = 30;
            int yloc = dim.height - 80;
            String ysfLabel2 = ySFLabel2 + "v/div";
            int string_width = getFontMetrics(g.getFont()).stringWidth(ysfLabel2);
            int string_height = getFontMetrics(g.getFont()).getHeight();
            g.clearRect(xloc, yloc, string_width, string_height);
            g.drawString(ysfLabel2, xloc, height + yloc);
        }

        private void drawYScaleLabel(Graphics g) {
            g.setColor(Color.red);
            int height = 10;
            int xloc = 20;
            int yloc = dim.height - 60;
            String ysfLabel = ySFLabel + "v/div";
            int string_width = getFontMetrics(g.getFont()).stringWidth(ysfLabel);
            int string_height = getFontMetrics(g.getFont()).getHeight();
            g.clearRect(xloc, yloc, string_width, string_height);
            g.drawString(ysfLabel, xloc, height + yloc);
        }
    }

    public static void main(String args[]) {
        OscopePanel osp = new OscopePanel();
        gui.ClosableJFrame cf = new gui.ClosableJFrame();
        Container c = cf.getContentPane();
        c.add(osp);
        c.setLayout(new GridLayout(1, 0));
        cf.setSize(400, 400);
        cf.show();
    }
}

